#!/bin/bash


#Company: Trenz Electronic GmbH
#Engineer: Mohsen Chamanbaz



#check tools
TOOLS=( \
    dd                  \
    losetup             \
    parted              \
    lsblk               \
    mkfs.vfat           \
    mkfs.ext4           \
    debootstrap         \
    gzip                \
    cpio                \
    sync                \
    chroot              \
    apt-get             \
    dpkg-reconfigure    \
    sed                 \
    locale-gen          \
    update-locale       \
    qemu-arm-static     \
)
    
STOP=0    
for (( i = 0; i < ${#TOOLS[@]}; i++ )); do
    TOOL=`which ${TOOLS[$i]}`
    if [ -z ${TOOL} ]; then
        echo -e "${TOOLS[$i]} ... \e[91mMISSING\e[0m"
        STOP=1
    else
        echo -e "${TOOLS[$i]} ... \e[92mINSTALLED\e[0m"    
    fi
done

if [ $STOP -eq 1 ]; then 
    echo "stop"
    exit 
fi

export LANG=en_US.UTF-8
export LC_ALL="en_US.UTF-8"
export LC_CTYPE="en_US.UTF-8"
export LANGUAGE="en_US.UTF-8"

petadir=.
image=te0808-debian.img
#size=1792
#size=7168
size=12000


# create an empty image
dd if=/dev/zero of=$image bs=1M count=$size status=progress

#get a free loop device
device=`losetup -f`
echo "++ Setting up device: $device"
losetup $device $image

boot_dir=/tmp/BOOT
root_dir=/tmp/ROOT

# Debian mirror list: https://www.debian.org/mirror/list
#mirror=http://ftp.uni-stuttgart.de/debian
mirror=http://ftp.cz.debian.org/debian
# distro=jessie
distro=stretch
#distro=buster
arch=arm64

passwd=root
timezone=Europe/Berlin

# Create partitions

parted -s $device mklabel msdos
parted -s $device mkpart primary fat32 4MB 256MB
parted -s $device mkpart primary ext4 256MB 100%

# get device names from the sorted list (-x NAME) !!!
boot_dev=/dev/`lsblk -lno NAME -x NAME $device | sed '2!d'`
root_dev=/dev/`lsblk -lno NAME -x NAME $device | sed '3!d'`

# Create file systems

mkfs.vfat -v $boot_dev
mkfs.ext4 -F -j $root_dev

# Mount file system

mkdir -p $boot_dir $root_dir

mount $boot_dev $boot_dir
mount $root_dev $root_dir

# Copy files to the boot file system

#cp $petadir/images/linux/image.ub $boot_dir
cp $petadir/images/linux/Image $boot_dir
cp $petadir/images/linux/system.dtb $boot_dir


#Generating Device Tree blob file for  rootfile system on Harddisk (system_harddisk.dtb)
dtc -I dtb -O dts -o $boot_dir/system_sd.dts $boot_dir/system.dtb
dtc -I dts -O dtb -o $boot_dir/system_sd.dtb $boot_dir/system_sd.dts	
sed -i s/mmcblk1p2/sda/g $boot_dir/system_sd.dts
dtc -I dts -O dtb -o $boot_dir/system_harddisk.dtb $boot_dir/system_sd.dts
rm -r $boot_dir/system_sd.dts
rm -r $boot_dir/system.dtb

# Install Debian base system to the root file system

debootstrap --foreign --arch $arch $distro $root_dir $mirror

# Add missing configuration files and packages

cp /etc/resolv.conf $root_dir/etc/
cp /usr/bin/qemu-aarch64-static $root_dir/usr/bin/

mkdir -p $root_dir/usr/local/bin
mkdir -p $root_dir/usr/local/sbin

mkdir -p $root_dir/lib/modules

# TE audio for TE0803
# cp -av $petadir/build/tmp/sysroots/plnx_arm/lib/modules/4.14.0-xilinx-v2018.2/ $root_dir/lib/modules
#cp -av $petadir/build/tmp/sysroots/plnx_arm/lib/modules/4.14.0-xilinx-v2018.3/ $root_dir/lib/modules
#cp -av $petadir/build/tmp/sysroots/plnx_arm/lib/modules/4.19.0-xilinx-v2019.2/ $root_dir/lib/modules
# get Xilinx modules

startdir=`pwd`
cd $petadir/build/tmp/deploy/images/plnx-zynqmp/
# gzip -dk --force petalinux-user-image-plnx-zynqmp.cpio.gz
mkdir tmp
cd tmp
cpio -id < ../petalinux-user-image-plnx-zynqmp.cpio
#cp -av lib/modules/4.14.0-xilinx-v2018.2/ $root_dlibgdk-pixbuf2.0-0

ir/lib/modules
#cp -av lib/modules/4.14.0-xilinx-v2018.3/ $root_dir/lib/modules
cp -av lib/modules/4.19.0-xilinx-v2019.2/ $root_dir/lib/modules

sync
cd ..
rm -rf tmp
# rm -f petalinux-user-image-plnx-zynqmp.cpio
cd $startdir


modules_path=`find build/tmp/work/plnx_zynqmp-xilinx-linux/linux-xlnx/ -maxdepth 5 -type d -print | grep image/lib/modules/4.19.0-xilinx-v2019.2`
cp -av $petadir/$modules_path/ $root_dir/lib/modules

X11DIR=$root_dir/usr/share/X11/xorg.conf.d
mkdir -p $X11DIR
cat << EOF_FBDEV > $X11DIR/99-fbdev.conf
Section "Device"  
  Identifier "myfb"
  Driver "fbdev"
  Option "fbdev" "/dev/fb0"
EndSection
EOF_FBDEV
#echo te-audio-codec >> $root_dir/etc/modules


chroot $root_dir <<- EOF_CHROOT
export LANG=C
export LC_ALL=C

/debootstrap/debootstrap --second-stage

cat <<- EOF_CAT > etc/apt/sources.listSoftware Design -  PetaLinux

For PetaLinux installation and  project creation, follow instructions from:

    PetaLinux KICKstart

Commands and actions used to create petalinux project

    petalinux-create -t project -n petalinux --template zynq
    copy hdf file to petalinux directory
    cd petalinux
    petalinux-config --get-hw-description
    leave default configuration and select "<Exit>"
    petalinux-create -t apps -n startup --enable
    copy/create startup app files
    petalinux-create -t apps -n rpicam --enable
    copy/create rpicam app files
    petalinux-create -t modules -n te-audio-codec --enable
    copy/create te-audio-codec module files
    copy/modify project-spec/meta-user/recipes-bsp/device-tree/files/system-user.dtsi
    petalinux-config -c rootfs
    Enable Filesystem Packages → Base → i2c-tools → i2c-tools and other apps needed for your system
    Save configuration and exit config
    petalinux-config -c kernel
        CONFIG_USB_USBNET
        CONFIG_USB_NET_SMSC95XX
        CONFIG_USBIP_CORE
        CONFIG_FB_SIMPLE
        SND_SIMPLE_CARD
        CONFIG_SND_SOC_ADI_AXI_I2S
    petalinux-build
deb $mirror $distro main contrib non-free
deb-src $mirror $distro main contrib non-free
deb $mirror $distro-updates main contrib non-free
deb-src $mirror $distro-updates main contrib non-free
deb http://security.debian.org/debian-security $distro/updates main contrib non-free
deb-src http://security.debian.org/debian-security $distro/updates main contrib non-free
EOF_CAT

cat <<- EOF_CAT > etc/apt/apt.conf.d/99norecommends
APT::Install-Recommends "0";
APT::Install-Suggests "0";
EOF_CAT

cat <<- EOF_CAT > etc/fstab
# /etc/fstab: static file system information.
# <file system> <mount point>               <type>    <options>      <dump> <pass>
#/dev/mmcblk1p2 /dev/disk/by-label/ROOTFS    ext4  errors=remount-ro    0       1		
#/dev/mmcblk1p1 /dev/disk/by-label/BOOT      vfat     defaults          0       2
/dev/mmcblk1p2 /media/ROOTFS                 ext4  errors=remount-ro    0       1		
/dev/mmcblk1p1 /media/BOOT                   vfat     defaults          0       2		
EOF_CAT

cat <<- EOF_CAT >> etc/securetty

# Serial Console(s) for Xilinx Zynq Ultrascale+
ttyPS0
ttyPS1
EOF_CAT

echo zynqmp > etc/hostname

apt-get update
apt-get -y upgrade

apt-get -y install locales

sed -i "/^# en_US.UTF-8 UTF-8$/s/^# //" etc/locale.gen
locale-gen
update-locale LANG=en_US.UTF-8

echo $timezone > etc/timezone
dpkg-reconfigure --frontend=noninteractive tzdata

apt-get -y install gedit mplayer openssh-server ca-certificates ntp ntpdate fake-hwclock \
  usbutils psmisc lsof parted curl wget vim ifplugd \
  python3 python3-numpy build-essential libfftw3-dev python3-scipy \
  xfonts-base tightvncserver xfce4-panel xfce4-session xfwm4 xfdesktop4 \
  xfce4-terminal thunar gnome-icon-theme \
  tcpdump ethtool net-tools picocom joe mc usbutils \
  xserver-xorg xinit xserver-xorg-video-fbdev \
  nodejs mousepad scilab-cli
#   mono-devel nodejs

# enable root login via ssh
sed -i 's/^#PermitRootLogin.*/&\nPermitRootLogin yes/' etc/ssh/sshd_config

touch etc/udev/rules.d/75-persistent-net-generator.rules
# --------------------------
#enable dhcp
#touch etc/network/interfaces.d
#cat <<- EOF_CAT > etc/network/interfaces.d/eth0libgdk-pixbuf2.0-0

touch etc/network
cat <<- EOF_CAT > etc/network/interfaces

auto lo
iface lo inet loopback

auto eth0
allow-hotplug eth0
iface eth0 inet dhcp

EOF_CAT
# --------------------------


#---------------------------
#Automatic Startx
touch /root/
cat <<- EOF_CAT > /root/.profile

# ~/.profile: executed by Bourne-compatible login shells.

if [ "$BASH" ]; then
  if [ -f ~/.bashrc ]; then
    . ~/.bashrc
  fi
fi

mesg n || true

#exec startx



EOF_CAT
#----------------------------


cat <<- EOF_CAT > etc/default/ifplugd
INTERFACES="eth0"
HOTPLUG_INTERFACES=""
ARGS="-q -f -u0 -d10 -w -I"
SUSPEND_ACTION="stop"
EOF_CAT

apt-get clean

echo root:$passwd | chpasswd

service ntp stop

history -c
EOF_CHROOT

rm $root_dir/etc/resolv.conf
rm $root_dir/usr/bin/qemu-aarch64-static

sync

# Unmount file systems
umount -f $boot_dir $root_dir

sleep 1

# rmdir $boot_dir $root_dir
rm -rf $boot_dir $root_dir

losetup -d $device

sync


